Package: simsem
Type: Package
Title: SIMulated Structural Equation Modeling
Version: 0.5-17
Date: 2025-04-02
Authors@R: c(person(given = "Sunthud", family = "Pornprasertmanit", role = "aut", email = "psunthud@gmail.com"), 
		         person(given = "Patrick", family = "Miller", role = "aut", email="pmille13@nd.edu"), 
		         person(given = c("Alexander", "M."), family = "Schoemann", role = "aut", email="schoemanna@ecu.edu", comment = c(ORCID = "0000-0002-8479-8798")), 
		         person(given = c("Terrence","D."), family = "Jorgensen", role = c("aut", "cre"), email="TJorgensen314@gmail.com", comment = c(ORCID = "0000-0001-5111-6773")),
             person(given = "Corbin", family = "Quick", role = "ctb", email="qcorbin@hsph.harvard.edu"))
Maintainer: Terrence D. Jorgensen <TJorgensen314@gmail.com>
Depends: R(>= 3.4), methods, lavaan(>= 0.6-12), utils, stats, graphics
Suggests: parallel, quantreg, splines, KernSmooth, semTools(>= 0.5-7),
        lavaan.mi, OpenMx(>= 2.0.0), copula
Description: Provides an easy framework for Monte Carlo simulation in structural equation modeling, which can be used for various purposes, such as such as model fit evaluation, power analysis, or missing data handling and planning. 
License: GPL (>= 2)
LazyLoad: yes
URL: https://simsem.org/
NeedsCompilation: no
Packaged: 2025-04-02 12:11:36 UTC; terrence
Author: Sunthud Pornprasertmanit [aut],
  Patrick Miller [aut],
  Alexander M. Schoemann [aut] (<https://orcid.org/0000-0002-8479-8798>),
  Terrence D. Jorgensen [aut, cre]
    (<https://orcid.org/0000-0001-5111-6773>),
  Corbin Quick [ctb]
Repository: CRAN
Date/Publication: 2025-04-02 12:30:01 UTC
Built: R 4.6.0; ; 2026-04-27 12:50:26 UTC; unix
