
importFrom("methods", show, is, new, slot, getMethod, setMethod)
importFrom("stats",
           vcov, cor, cov, var, sd, cov2cor, uniroot, nlminb,
           rnorm, runif, qnorm, qchisq, pchisq, quantile, glm, predict, fitted,
           density, loess.smooth, binomial)
importFrom("utils",
           data, capture.output, write.table, combn)
importFrom("graphics", hist, plot, contour, persp, par,
           abline, lines, points, barplot, legend)

importFrom("lavaan", anova, summary, coef, cor2cov, lavInspect, parTable)

exportClasses(SimMatrix,
	SimVector,
	SimSem,
	SimResult,
	SimDataDist,
	SimMissing)

exportMethods(summaryShort, summary, coef, anova, inspect)

export(bindDist, rawDraw,
	model, model.cfa, model.path, model.sem,
	estmodel, estmodel.cfa, estmodel.path, estmodel.sem,
	model.lavaan,
	impose, imposeMissing,
	summaryFit, summaryParam, summaryMisspec, summaryConverge, summarySeed, summaryTime,
	getCutoff, plotCutoff,
	findRecursiveSet, findPossibleFactorCor, findIndIntercept,
	findIndResidualVar, findFactorIntercept, findFactorResidualVar,
	findIndTotalVar, findIndMean, findFactorTotalVar, findFactorTotalCov, findFactorMean,
	getPower, plotPower, findPower, plotMisfit, getExtraOutput,
	popMisfitMACS,
	popDiscrepancy,
	continuousPower, combineSim,
	getCIwidth, plotCIwidth,
	getCoverage, continuousCoverage, plotCoverage, findCoverage,
	getPowerFit, getPowerFitNested, getPowerFitNonNested,
	plotPowerFit, plotPowerFitNested, plotCutoffNested, getCutoffNested, multipleAllEqual,
	plotDist,
	summaryPopulation, getPopulation, setPopulation,
	getCutoffNonNested, plotCutoffNonNested, plotPowerFitNonNested,
	likRatioFit, # This function (and others in same file) are NEVER used in this package. Why do they exist?
	pValue, pValueNested, pValueNonNested,
	bind, binds, generate, analyze, sim, draw, createData, miss, exportData,
	plotLogitMiss)
